package com.jpattern.orm;

import static org.junit.Assert.*;
import java.io.File;
import java.io.FileInputStream;
import java.math.BigDecimal;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Rule;
import org.junit.rules.TestName;
import org.junit.runner.RunWith;
import org.junit.runners.BlockJUnit4ClassRunner;

import com.jpattern.logger.ILogger;
import com.jpattern.orm.connection.ConnectionWrapper;
import com.jpattern.orm.dialect.IDialect;
import com.jpattern.orm.generator.IGeneratorStrategy;
import com.jpattern.orm.logger.OrmLogger;
import com.jpattern.orm.script.IScriptExecutor;
import com.jpattern.orm.session.ISession;
import com.jpattern.orm.session.ISessionProvider;
import com.jpattern.orm.test.IJpOrmTestsConstants;
import com.jpattern.orm.transaction.ITransaction;
import com.jpattern.shared.util.Chronometer;

/**
 * 
 * @author Francesco Cina
 *
 * 20/mag/2011
 */
@RunWith(BlockJUnit4ClassRunner.class)
public abstract class SharedBaseTest {

	@Rule public final TestName name = new TestName();
	private final Chronometer chronometer = new Chronometer();
	private ILogger logger;
	private static ISessionProvider sessionProvider;
	private static IDialect dialect;
	private static IGeneratorStrategy generatorStrategy;

	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
	}

	@AfterClass
	public static void tearDownAfterClass() throws Exception {
	}

	@Before
	public void setUpBeforeTest() throws Exception {

		chronometer.restart();

		logger = OrmLogger.getOrmLogger(getClass());

		logger.info("setUpBeforeTest","===================================================================");
		logger.info("setUpBeforeTest","BEGIN TEST " + name.getMethodName());
		logger.info("setUpBeforeTest","OPEN_CONNECTIONS: " + ConnectionWrapper.OPEN_CONNECTIONS );
		logger.info("setUpBeforeTest","CLOSED_CONNECTIONS: " + ConnectionWrapper.CLOSED_CONNECTIONS );
		logger.info("setUpBeforeTest","TOTAL_CONNECTIONS: " + ConnectionWrapper.TOTAL_CONNECTIONS );
		logger.info("setUpBeforeTest","COMMITTED_CONNECTIONS: " + ConnectionWrapper.COMMITTED_CONNECTIONS );
		logger.info("setUpBeforeTest","ROLLBACKED_CONNECTIONS: " + ConnectionWrapper.ROLLBACKED_CONNECTIONS );
		logger.info("setUpBeforeTest","===================================================================");

		setUp();

	}

	@After
	public void tearDownAfterTest() throws Exception {
		tearDown();

		chronometer.pause();
		final String time = new BigDecimal(chronometer.read() ).divide(new BigDecimal(1000)).toString();

		logger.info("tearDownAfterTest","===================================================================");
		logger.info("tearDownAfterTest","END TEST " + name.getMethodName());
		logger.info("tearDownAfterTest","OPEN_CONNECTIONS: " + ConnectionWrapper.OPEN_CONNECTIONS );
		logger.info("tearDownAfterTest","CLOSED_CONNECTIONS: " + ConnectionWrapper.CLOSED_CONNECTIONS );
		logger.info("tearDownAfterTest","TOTAL_CONNECTIONS: " + ConnectionWrapper.TOTAL_CONNECTIONS );
		logger.info("tearDownAfterTest","COMMITTED_CONNECTIONS: " + ConnectionWrapper.COMMITTED_CONNECTIONS );
		logger.info("tearDownAfterTest","ROLLBACKED_CONNECTIONS: " + ConnectionWrapper.ROLLBACKED_CONNECTIONS );
		logger.info("tearDownAfterTest","Execution time: " + time + " seconds");
		logger.info("tearDownAfterTest","===================================================================");

		assertEquals( ConnectionWrapper.TOTAL_CONNECTIONS , ConnectionWrapper.CLOSED_CONNECTIONS);
	}

	protected abstract void setUp() throws Exception;
	protected abstract void tearDown() throws Exception;

	protected String getTestInputBasePath() throws Exception {
		return IJpOrmTestsConstants.TEST_FILE_INPUT_BASE_PATH;
	}

	protected String getTestOutputBasePath() throws Exception {
		mkDir(IJpOrmTestsConstants.TEST_FILE_OUTPUT_BASE_PATH);
		return IJpOrmTestsConstants.TEST_FILE_OUTPUT_BASE_PATH;
	}

	protected void mkDir( String dirPath ) throws Exception {
		final File path = new File(dirPath);
		if (!path.exists()) {
			path.mkdirs();
		}
	}

	public static void init(ISessionProvider sessionProvider, String createScriptFileName, String dropScriptFileName, IDialect dialect, IGeneratorStrategy generatorStrategy) throws Exception {
		SharedBaseTest.generatorStrategy = generatorStrategy;
		SharedBaseTest.sessionProvider = sessionProvider;
		SharedBaseTest.dialect = dialect;
		assertNotNull(sessionProvider);
		final IJPOrm jpOrm = new JPOrm(sessionProvider, dialect, generatorStrategy);
		final ISession session = jpOrm.session();
		final ITransaction tx = session.transaction();
		final IScriptExecutor scriptExecutor = session.scriptExecutor();
		try {
			scriptExecutor.execute(new FileInputStream(dropScriptFileName));
		} catch (final Exception e) {}
		scriptExecutor.execute(new FileInputStream(createScriptFileName));
		tx.commit();
	}

	protected JPOrm getJPOrm() {
		return new JPOrm(sessionProvider, dialect, generatorStrategy);
	}

}

